#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#ifdef POSIX
#  include <sys/select.h>
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#ifdef STRICT_BSD
#include <strings.h>
#else
#include <string.h>
#endif
#include <ctype.h>
#include <signal.h>
#include "utility.h"


#ifdef POSIX
#define FNDELAY O_NDELAY
#define bcopy(x, y, n)  memcpy(y, x, n)
#endif

#ifndef FNDELAY
#define FNDELAY O_NDELAY
#endif

#ifndef INADDR_NONE
#define INADDR_NONE (int) -1
#endif

extern int die(), socket(), setsockopt(), bind(), listen(), accept(), recv(),
    send();

int
getsock(address, port)
    char *address;
    int port;
{
    int s;
    struct in_addr tmpaddr;
    struct hostent *tmphost;
    struct sockaddr_in saddr;

	/* get a fresh socket */
    if ((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	perror("lorien: Unable to get a new socket");
	die(errno);
    }

	/* decode address */
    if (isdigit(*address)) {
	if ((tmpaddr.s_addr = inet_addr(address)) == (u_short)INADDR_NONE) {
	    perror("lorien: Unable to figure out numeric address");
	    die(errno);
	}
    } else {
        alarm(1);
	if ((tmphost = gethostbyname(address)) == (struct hostent *)0) {
	    perror("lorien: unable to get host adress:");
	    die(9);
	}
        alarm(0);
	bcopy(tmphost->h_addr, (char *) &tmpaddr, sizeof(struct in_addr));
    }

	/* set up socket address */
    bcopy((char *) &tmpaddr, (char *) &(saddr.sin_addr), sizeof(struct in_addr));
    saddr.sin_addr.s_addr = INADDR_ANY;
    saddr.sin_family = AF_INET;
    saddr.sin_port = (unsigned short) htons((unsigned short) port);
    setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) 0, 0);

	/* attempt to bind the socket to our address */
    if (bind(s, (struct sockaddr *) &saddr, sizeof(struct sockaddr_in)) == -1) {
	perror("lorien: Error attempting to bind");
	die(errno);
    }

	/* set our backlog of waiting connections to 5(max the system allows) */
    if (listen(s, 5) == -1) {
	perror("lorien: Error listening");
	die(errno);
    }

    return s;
}

int
acceptcon(s, from, port)
    int s;
    char *from;
    int *port;
{
    int ns;
    int length = sizeof(struct sockaddr_in);
    char *buf;
    struct hostent *tmphost;
    struct sockaddr_in saddr;

    if ((ns = accept(s, (struct sockaddr *) &saddr, &length)) == -1 ) {
	perror("lorien: Error trying to accept connections");
	return -1;
	    /*die(errno);*/
    }

    fcntl(ns, F_SETFL, FNDELAY);

#ifndef SKIP_HOSTLOOKUP
    tmphost = gethostbyaddr((char *) &(saddr.sin_addr), sizeof(struct in_addr), AF_INET);
#endif
    if (tmphost == (struct hostent *)0) {
	perror("lorien: unable to get host adress");
	(void) strcpy(from, inet_ntoa(saddr.sin_addr));
    } else {
	buf = tmphost->h_name;
	if (buf != (char *)0) {
	    (void) strcpy(from, buf);
	} else
	    (void) strcpy(from, inet_ntoa(saddr.sin_addr));
    } 

    *port = (int)  ntohs((unsigned short) saddr.sin_port);

    return ns;
}

int
infromsock(s, buffer, size)
    int s;
    char *buffer;
    int size;
{
    int numchars;

    if ((numchars = recv(s, buffer, size-1, 0x0)) <= 0 ) { /* panic for now */
	perror("lorien: receive failed");
	return -1;
    }

    if (numchars == 0) { /* foreign end closed the connection be graceful */
	return -1; /* tell the calling proc to deal with it. */
    }
    buffer[numchars] = buffer[size - 1] = '\000';

    return 0;
}

int
outtosock(s, buffer)
    int s;
    char *buffer;
{
    int length, numsent;

    length = strlen(buffer);
    if ((numsent = send(s, buffer, length, 0x0)) == -1) {
	perror("lorien: send");
	fprintf(stderr, "Atempted send to socket %d with message \"%s\"\n",
		s, buffer);
	return -1;
    } else if (numsent < length) {
	fprintf(stderr, "lorien: send garbled lost characters.\n");
    }

    return 0;
}

